﻿using CyUSB;
using System;
using System.Collections.Generic;
using System.Diagnostics;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Navigation;
using System.Windows.Shapes;

namespace VoidVsNonVoidFunctionCallsWithUSBFS_CyNET
{
    /// <summary>
    /// Interaction logic for MainWindow.xaml
    /// </summary>
    public partial class MainWindow : Window
    {
        USBDeviceList usbDevices = null;
        CyUSBDevice MyCyUSBDevice = null;

        CyInterruptEndPoint inEndpoint = null;
        CyInterruptEndPoint outEndpoint = null;

        const int XFERSIZE_IN = 64;
        const int XFERSIZE_OUT = 64;
        byte[] inData = new byte[XFERSIZE_IN];
        byte[] outData = new byte[XFERSIZE_OUT];

        bool IsDeviceConnected = false;

        public MainWindow()
        {
            InitializeComponent();

            // Cypress USB :
            usbDevices = new USBDeviceList(CyConst.DEVICES_CYUSB);      // Create a list of CYUSB devices
            //Adding event handlers for device attachment and device removal
            usbDevices.DeviceAttached += new EventHandler(usbDevices_DeviceAttached);
            usbDevices.DeviceRemoved += new EventHandler(usbDevices_DeviceRemoved);
            IsDeviceConnected = setAndSearchDevice(); //The below function sets the device with particular VID and PId and searches for the device with the same VID and PID.
            if (IsDeviceConnected)
            {
                this.Title = MyCyUSBDevice.Product + " PSoC connected.";
            }
        }


        void usbDevices_DeviceAttached(object sender, EventArgs e)
        {
            IsDeviceConnected = setAndSearchDevice();
            if (IsDeviceConnected)
            {
                this.Title = MyCyUSBDevice.Product + " PSoC connected.";
            }

        }

        void usbDevices_DeviceRemoved(object sender, EventArgs e)
        {
            IsDeviceConnected = setAndSearchDevice();
            if (IsDeviceConnected == false)
            {
                this.Title = "PSoC DISconnected !";
            }
        }


        /* Summary
           The function sets the device, as the one having VID=04b4 and PID=00F3
           This will detect only the devices with the above VID, PID, Product combinations
            The function sets InterruptInEndPt and InterruptOutEndPt as well
       */
        public bool setAndSearchDevice()
        {
            bool IsSearchedDeviceFound = false;

            int nDeviceList = usbDevices.Count;
            for (int i = 0; i < nDeviceList; i++)
            {
                USBDevice fxDevice = usbDevices[i];

                // fxDevice.FriendlyName = "Cypress FX3 USB BootLoader Device"
                /*
                    VendorID = 1204 (0x4B4)
                    ProductID = 243 (0xF3)
                    Name = "ColorBot"
                    Product = "ColorBot"
                 */
                if (fxDevice.VendorID == 1204
                    && fxDevice.ProductID == 243
                    && fxDevice.Product == "ColorBot")
                {
                    MyCyUSBDevice = usbDevices[i] as CyUSBDevice;
                    inEndpoint = MyCyUSBDevice.InterruptInEndPt;
                    outEndpoint = MyCyUSBDevice.InterruptOutEndPt;

                    return true;
                }
            }

            MyCyUSBDevice = null;


            return IsSearchedDeviceFound;
        }


        // Accepted to sample command from PC: 'all' void and non void function call will happens
        enum Commands
        {
            cmd_BeepSOSX_VoidVersion = 1,
            cmd_BeepSOSX_NONVoidVersion_ButCalledItWithoutUseFakeVariableToStoreReturnValue,
            cmd_BeepSOSX_NONVoidVersion_CalledItWITHUseFakeVariableToStoreReturnValue
        };

        // Packet: Counter, Random
        UInt32 Counter = 0;
        UInt16 Random = 0;
        Random rnd = new Random();
        byte[] rnd2buffer = new byte[2];

        void SendPacket(Commands command,
                        uint inEndpointTimeOut = 10000  // default time-out
            )
        {
            if (IsDeviceConnected)
            {
                if (MyCyUSBDevice != null)
                {
                    if (MyCyUSBDevice.InterruptOutEndPt != null && MyCyUSBDevice.InterruptInEndPt != null)
                    {
                        var watch = Stopwatch.StartNew(); // debug time measurement /START

                        // Zeroing out and in buffers:
                        Array.Clear(outData, 0, outData.Length);
                        Array.Clear(inData, 0, inData.Length);

                        // SEND:
                        // Packet format: opener: '(' , close: ')'
                        outData[0] = (byte)'(';
                        outData[63] = (byte)')';
                        // outData[1-4] : unit32 Counter LSB ... Counter MSB
                        byte[] ba_Counter = BitConverter.GetBytes(Counter);
                        outData[1] = ba_Counter[0];
                        outData[2] = ba_Counter[1];
                        outData[3] = ba_Counter[2];
                        outData[4] = ba_Counter[3];

                        // outData[5-6] : unit16 Random LSB Random MSB
                        rnd.NextBytes(rnd2buffer);
                        Random = BitConverter.ToUInt16(rnd2buffer, 0);
                        outData[5] = rnd2buffer[0];
                        outData[6] = rnd2buffer[1];

                        // outData[7-8] : unit16 Command LSB Command MSB
                        UInt16 _command16 = (UInt16)command;
                        byte[] ba_Command = BitConverter.GetBytes(_command16);
                        outData[7] = ba_Command[0];
                        outData[8] = ba_Command[1];

                        // outData[9-56] : parameters, CLIPPED FROM THIS VERSION.
                        

                        int xferLen = XFERSIZE_OUT;

                        bool bResultT = outEndpoint.XferData(ref outData, ref xferLen); // „XferData‟ does not return until the full data is transferred. 

                        // RECEIVE:
                        xferLen = XFERSIZE_IN; // Returns true if the transaction successfully completes before TimeOut has elapsed
                                               // „XferData‟ does not return until the full data is transferred. 
                                               // In other words, the application is blocked until the full data transfer completes.
                        inEndpoint.TimeOut = inEndpointTimeOut; // default 10000 és a doskival szemben ms lehet
                        bool bResultR = inEndpoint.XferData(ref inData, ref xferLen); // ha xferLen nem a valós buffer méret, akkor false-ra fut

                        // parse:
                        UInt32 Counter_getBack = BitConverter.ToUInt32(inData, 1);
                        UInt16 Random_getBack = BitConverter.ToUInt16(inData, 5);
                        UInt16 command_getBack = BitConverter.ToUInt16(inData, 7);

                        // CLIPPED FROM THIS VERSION.
                        //byte CommandProcessed = inData[60];
                        //byte ErrorState = inData[61];
                        //byte BusyState = inData[62];

                        bool WellFormattedReply = false; 
                        if (inData[0] == '(' && inData[63] == ')')
                        {
                            WellFormattedReply = true;
                        }

                        bool CounterVSCounter_getBack_Matches = false;
                        if (Counter == Counter_getBack)
                        {
                            CounterVSCounter_getBack_Matches = true;
                        }

                        bool RandomVSRandom_getBack_Matches = false;
                        if (Random == Random_getBack)
                        {
                            RandomVSRandom_getBack_Matches = true;
                        }

                        // command:
                        bool commandVScommand_getBack_Matches = false;
                        var c1 = (int)command;
                        var c2 = (int)command_getBack;
                        if ((int)command == (int)command_getBack)
                        {
                            commandVScommand_getBack_Matches = true;
                        }

                        watch.Stop(); //  debug time measurement /END

                        DateTime _now = DateTime.Now;

                        TextBox_logOutPut.Text = "Tx succ: " + bResultT.ToString()
                            + " Rx succ: " + bResultR.ToString()
                            + ", Reply timeStamp: " + _now.Second.ToString() + " sec, " + _now.Millisecond.ToString() + " ms"

                            + "\n"
                            + System.Text.Encoding.ASCII.GetString(new[] { inData[0] }) // packet start  '('
                            + " "
                            + Counter_getBack.ToString()
                            + " "
                            + Random_getBack.ToString()
                            + " "
                            + command_getBack.ToString()
                            + " ... "
                            + System.Text.Encoding.ASCII.GetString(new[] { inData[63] }) // packet end ')'

                            + " \nWellFormattedReply?: " + WellFormattedReply.ToString()
                            + " \nCounterVSCounter_getBack_Matches?: " + CounterVSCounter_getBack_Matches.ToString()
                            + " \nRandomVSRandom_getBack_Matches?: " + RandomVSRandom_getBack_Matches.ToString()
                            + " \ncommandVScommand_getBack_Matches?: " + commandVScommand_getBack_Matches.ToString()

                            // CLIPPED FROM THIS VERSION.
                            //+ " \nCommandProcessed: " + CommandProcessed.ToString()
                            //+ " ErrorState: " + ErrorState.ToString()
                            //+ " BusyState: " + BusyState.ToString()

                            
                             + "\n\nElapsedMilliseconds: " + watch.ElapsedMilliseconds.ToString()
                            ;


                        Counter++;
                    }
                }
            }

        }

        private void Button_cmd_BeepSOSX_VoidVersion_Click(object sender, RoutedEventArgs e)
        {
            SendPacket(Commands.cmd_BeepSOSX_VoidVersion, inEndpointTimeOut: 10000);
        }

        private void Button_cmd_BeepSOSX_NONVoidVersion_ButCalledItWithoutUseFakeVariableToStoreReturnValue_Click(object sender, RoutedEventArgs e)
        {
            SendPacket(Commands.cmd_BeepSOSX_NONVoidVersion_ButCalledItWithoutUseFakeVariableToStoreReturnValue, inEndpointTimeOut: 10000);
        }

        private void Button_cmd_BeepSOSX_NONVoidVersion_CalledItWITHUseFakeVariableToStoreReturnValue_Click(object sender, RoutedEventArgs e)
        {
            SendPacket(Commands.cmd_BeepSOSX_NONVoidVersion_CalledItWITHUseFakeVariableToStoreReturnValue, inEndpointTimeOut: 10000);
        }


    }
}
